@@ -16,11 +16,16 @@
 #include <linc/linc-connection.h>
 
 #include "linc-private.h"
-#include "linc-debug.h"
 
 #undef LOCAL_DEBUG
 
+#ifdef G_OS_WIN32
+#  include <io.h>
+#  define mkdir(path, mode) _mkdir (path)
+#endif
+
 static char *link_tmpdir = NULL;
+static gboolean use_local_host = FALSE;
 
 /*
  * make_local_tmpdir:
@@ -47,7 +52,7 @@ make_local_tmpdir (const char *dirname)
 			if (stat (dirname, &statbuf) != 0)
 				g_error ("Can not stat %s\n", dirname);
 
-#ifndef __CYGWIN__
+#if !defined (__CYGWIN__) && !defined(_WIN32)
 			if (statbuf.st_uid != getuid ())
 				g_error ("Owner of %s is not the current user\n", dirname);
 
@@ -64,11 +69,13 @@ make_local_tmpdir (const char *dirname)
 		}
 	}
 
+#if defined (HAVE_UTIME_H) || defined (HAVE_SYS_UTIME_H)
 	{ /* Hide some information ( apparently ) */
 		struct utimbuf utb;
 		memset (&utb, 0, sizeof (utb));
 		utime (dirname, &utb);
 	}
+#endif
 }
 
 /**
@@ -128,6 +135,9 @@ link_get_local_hostname (void)
 {
 	static char local_host[NI_MAXHOST] = { 0 };
 
+	if (use_local_host)
+		return "localhost";
+
 	if (local_host [0])
 		return local_host;
 
@@ -137,6 +147,12 @@ link_get_local_hostname (void)
 	return local_host;
 }
 
+void
+link_use_local_hostname (gboolean use)
+{
+        use_local_host = use;
+}
+
 /*
  * True if succeeded in mapping, else false.
  */
@@ -545,31 +561,6 @@ link_protocol_get_sockaddr_unix (const LinkProtocolInfo *proto,
 }
 #endif /* AF_UNIX */
 
-/*
- * link_protocol_get_sockaddr_irda:
- * @proto:
- * @hostname:
- * @service:
- * @saddr_len:
- *
- * NOTE: This function is not implemented. We need to hack something
- *       together from irda_getaddrinfo.
- *
- * Return Value:
- */
-#ifdef AF_IRDA
-static struct sockaddr *
-link_protocol_get_sockaddr_irda (const LinkProtocolInfo *proto,
-				 const char             *hostname,
-				 const char             *service,
-				 LinkSockLen            *saddr_len)
-{
-	g_assert (proto->family == AF_IRDA);
-
-	return NULL;
-}
-#endif /* AF_IRDA */
-
 /*
  * link_protocol_get_sockaddr:
  * @proto: a #LinkProtocolInfo structure.
@@ -797,28 +788,6 @@ link_protocol_get_sockinfo_unix (const LinkProtocolInfo  *proto,
 }
 #endif /* AF_UNIX */
 
-/*
- * link_protocol_get_sockinfo_irda:
- * @proto: a #LinkProtocolInfo structure.
- * @sockaddr: a #sockaddr_irda structure desribing the socket.
- * @hostname: 
- * @service: 
- *
- * Return Value: #TRUE if the function succeeds, #FALSE otherwise.
- */
-#ifdef AF_IRDA
-static gboolean
-link_protocol_get_sockinfo_irda (const LinkProtocolInfo  *proto,
-				 const struct sockaddr   *saddr,
-				 gchar                  **hostname,
-				 gchar                  **portnum)
-{
-	g_assert (proto && saddr && saddr->sa_family == AF_IRDA);
-
-	return FALSE;
-}
-#endif /* AF_IRDA */
-
 /*
  * link_protocol_get_sockinfo:
  * @proto: a #LinkProtocolInfo structure.
@@ -920,7 +889,7 @@ link_protocol_tcp_setup (int                   fd,
 			return;
 
 		setsockopt (fd, proto->p_proto, TCP_NODELAY, 
-		            &on, sizeof (on));
+		            (const char *) &on, sizeof (on));
 	}
 #endif
 }
@@ -933,7 +902,12 @@ static LinkProtocolInfo static_link_protocols[] = {
 	AF_INET, 			/* family */
 	sizeof (struct sockaddr_in), 	/* addr_len */
 	IPPROTO_TCP, 			/* stream_proto_num */
-	0, 				/* flags */
+					/* flags */
+#ifdef G_OS_WIN32
+	LINK_PROTOCOL_NEEDS_BIND,
+#else
+	0,
+#endif
 	link_protocol_tcp_setup, 	/* setup */
 	NULL, 				/* destroy */
 	link_protocol_get_sockaddr_ipv4,/* get_sockaddr */
@@ -968,216 +942,10 @@ static LinkProtocolInfo static_link_protocols[] = {
 	link_protocol_get_sockinfo_unix, 		/* get_sockinfo */
 	link_protocol_unix_is_local                     /* is_local */
 	},
-#endif
-#ifdef AF_IRDA
-	{
-	"IrDA", 				/* name */
-	AF_IRDA, 				/* family */
-	sizeof (struct sockaddr_irda), 		/* addr_len */
-	0, 					/* stream_proto_num */
-	LINK_PROTOCOL_NEEDS_BIND, 		/* flags */
-	NULL, 					/* setup */
-	NULL, 					/* destroy */
-	link_protocol_get_sockaddr_irda, 	/* get_sockaddr */
-	link_protocol_get_sockinfo_irda, 	/* get_sockinfo */
-	NULL                                    /* is_local */
-	},
 #endif
 	{ NULL /* name */ }
 };
 
-/* 
- * Routines for AF_IRDA 
- * FIXME: These are left here only as a reference for implementing
- *        link_protocol_get_sockinfo_irda and 
- *        link_protocol_get_sockaddr_irda
- *       
- */
-#if 0
-#ifdef AF_IRDA
-
-#define MAX_IRDA_DEVICES 10
-#define IRDA_NICKNAME_MAX (sizeof (((struct irda_device_info *)NULL)->info) + 1)
-
-static int
-irda_find_device (guint32  *addr,
-		  char     *name,
-		  gboolean  name_to_addr)
-{
-	struct irda_device_list *list;
-	unsigned char           *buf;
-	int                      len, i, retval, fd;
-
-	retval = -1;
-
-	fd = socket (AF_IRDA, SOCK_STREAM, 0);
-	if (fd < 0)
-		return -1;
-
-	len = sizeof (struct irda_device_list) +
-	      sizeof (struct irda_device_info) * MAX_IRDA_DEVICES;
-
-	buf = g_alloca (len);
-	list = (struct irda_device_list *)buf;
-        
-	if (getsockopt (fd, SOL_IRLMP, IRLMP_ENUMDEVICES, buf, &len))
-		goto out;
-
-	if (len < 1)
-		goto out;
-
-	for (i = 0; i < list->len && retval; i++) {
-		if(name_to_addr) {
-			if (!strcmp (list->dev[i].info, name)) {
-				*addr = list->dev[i].daddr;
-				retval = 0;
-			}
-		}
-		else {
-			if (list->dev[i].daddr == *addr) {
-				strncpy (name, list->dev[i].info,
-					 sizeof(list->dev[i].info));
-				name[sizeof (list->dev[i].info)] = '\0';
-				retval = 0;
-			}
-		}
-	}
-
- out:
-	LINK_CLOSE (fd);
-
-	return retval;
-}
-
-#define IRDA_PREFIX      "IrDA-"
-#define IRDA_PREFIX_LEN  5
-
-static int
-irda_getaddrinfo (const char             *nodename,
-		  const char             *servname,
-		  const struct addrinfo  *hints,
-		  struct addrinfo       **res)
-{
-	struct sockaddr_irda  sai;
-	struct addrinfo      *retval;
-	char                  hnbuf[IRDA_NICKNAME_MAX + IRDA_PREFIX_LEN];
-	int                   n;
-	char                 *tptr;
-
-	/* 
-	 * For now, it *has* to start with IRDA_PREFIX to be in the IRDA
-	 * hostname/address format we use 
-	 */
-	if (nodename && strcmp (nodename, IRDA_PREFIX))
-		return EAI_NONAME;
-
-	sai.sir_family   = AF_IRDA;
-	sai.sir_lsap_sel = LSAP_ANY;
-
-	if (servname)
-		g_snprintf (sai.sir_name, sizeof (sai.sir_name), "%s", servname);
-	else {
-		struct timeval t;
-
-		gettimeofday (&t, NULL);
-		g_snprintf (sai.sir_name, sizeof (sai.sir_name), "IIOP%x%x",
-		            rand(), (guint)(t.tv_sec^t.tv_usec));
-	}
-
-	if (nodename) {
-		if (!strncmp (nodename + IRDA_PREFIX_LEN, "0x", 2)) {
-			if (sscanf (nodename + strlen(IRDA_PREFIX "0x"),
-				    "%u", &sai.sir_addr) != 1)
-				return EAI_NONAME;
-
-			/* It's a numeric address - we need to find the hostname */
-			g_strncpy (hnbuf, IRDA_PREFIX, IRDA_NICKNAME_MAX + IRDA_PREFIX_LEN);
-			if (irda_find_device (&sai.sir_addr, 
-					      hnbuf + IRDA_PREFIX_LEN,
-					      FALSE))
-				return EAI_NONAME;
-
-			nodename = hnbuf;
-		}
-		else if (!(hints->ai_flags & AI_NUMERICHOST)) {
-			/* It's a name - we need to find the address */
-			if (irda_find_device (&sai.sir_addr, 
-					      (char *)nodename + IRDA_PREFIX_LEN,
-					      TRUE))
-				return EAI_NONAME;
-		}
-		else
-			return EAI_NONAME;
-	}
-	else
-		/* AI_PASSIVE flag gets ignored, sort of */
-		hnbuf[0] = 0;
-
-	n = sizeof (struct addrinfo) + sizeof (struct sockaddr_irda);
-
-	if (hints->ai_flags & AI_CANONNAME)
-		n += strlen(hnbuf) + 1;
-
-	retval = g_malloc0(n);
-
-	tptr = (char *)retval;
-	tptr += sizeof (struct addrinfo);
-	retval->ai_addr = (struct sockaddr *)tptr;
-	memcpy (retval->ai_addr, &sai, sizeof (struct sockaddr_irda));
-	tptr += sizeof (struct sockaddr_irda);
-	g_strncpy (tptr, hnbuf, IRDA_NICKNAME_MAX + IRDA_PREFIX_LEN);
-	retval->ai_family = AF_IRDA;
-	retval->ai_socktype = SOCK_STREAM;
-	retval->ai_protocol = 0;
-	retval->ai_next = NULL;
-	retval->ai_addrlen = sizeof(struct sockaddr_irda);
-
-	*res = retval;
-
-	return 0;
-}
-
-static int
-irda_getnameinfo (const struct sockaddr *sa,
-		  LinkSockLen            sa_len,
-		  char                  *host,
-		  size_t                 hostlen,
-		  char                  *serv,
-		  size_t                 servlen,
-		  int                    flags)
-{
-	struct sockaddr_irda *sai = (struct sockaddr_irda *) sa;
-	gboolean got_host = FALSE;
-	/* Here, we talk to the host specified, and ask it for its name */
-
-	if (sa_len != sizeof (struct sockaddr_irda))
-		return -1;
-
-	/* It doesn't seem like the sir_lsap_sel is supposed to be taken into consideration when connecting... */
-	if (!(flags & NI_NUMERICHOST)) {
-		guint32 daddr;
-		char hostbuf [IRDA_NICKNAME_MAX];
-
-		daddr = sai->sir_addr;
-		if (!irda_find_device (&daddr, hostbuf, FALSE)) {
-			g_snprintf (host, hostlen, "%s", hostbuf);
-			got_host = TRUE;
-		}
-	}
-	if (!got_host) {
-		if (flags & NI_NAMEREQD)
-			return -1;
-
-		g_snprintf (host, hostlen, IRDA_PREFIX "%#08x", sai->sir_addr);
-	}
-
-	g_snprintf(serv, servlen, "%s", sai->sir_name);
-
-	return 0;
-}
-#endif /* AF_IRDA */
-#endif /* 0 */
-
 void
 link_protocol_destroy_cnx (const LinkProtocolInfo *proto,
 			   int                     fd,
@@ -1189,8 +957,8 @@ link_protocol_destroy_cnx (const LinkProtocolInfo *proto,
 	if (fd >= 0) {
 		if (proto->destroy)
 			proto->destroy (fd, host, service);
-		
-		LINK_CLOSE (fd);
+		d_printf ("link_protocol_destroy_cnx: closing %d\n", fd);
+		LINK_CLOSE_SOCKET (fd);
 	}
 }
 
@@ -1211,7 +979,8 @@ link_protocol_destroy_addr (const LinkProtocolInfo *proto,
 			proto->destroy (fd, NULL, addr_un->sun_path);
 		}
 #endif
-		LINK_CLOSE (fd);
+		d_printf ("link_protocol_destroy_addr: closing %d\n", fd);
+		LINK_CLOSE_SOCKET (fd);
 		g_free (saddr);
 	}
 
