@@ -61,13 +61,13 @@ typedef struct {
 } TkInfo;
 
 #define DEF_TC_BASIC(nom, c_align)                                      \
-	const struct CORBA_TypeCode_struct TC_CORBA_##nom##_struct = {  \
+	struct CORBA_TypeCode_struct TC_CORBA_##nom##_struct = {	\
 		{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},           \
 		CORBA_tk_##nom,                                         \
 		0, 0, c_align,						\
 		0, 0, NULL, NULL,					\
 		#nom,                                                   \
-		"IDL:CORBA/" #nom ":1.0",                               \
+		"IDL:omg.org/CORBA/" #nom ":1.0",			\
 		NULL, CORBA_OBJECT_NIL, -1				\
 }
 
@@ -78,16 +78,16 @@ typedef struct {
 #define CORBA_tk_unsigned_short CORBA_tk_ushort
 #define CORBA_tk_long_double CORBA_tk_longdouble
 
-const struct CORBA_TypeCode_struct TC_null_struct = {
+struct CORBA_TypeCode_struct TC_null_struct = {
 	{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
 	CORBA_tk_null, 0, 0, -1, 0, 0, NULL,
-	CORBA_OBJECT_NIL, "null", "Null"
+	CORBA_OBJECT_NIL, "null", "IDL:omg.org/CORBA/Null:1.0"
 };
 
-const struct CORBA_TypeCode_struct TC_void_struct = {
+struct CORBA_TypeCode_struct TC_void_struct = {
 	{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
-	CORBA_tk_null, 0, 0, -1, 0, 0, NULL,
-	CORBA_OBJECT_NIL, "void", "IDL:CORBA/void:1.0"
+	CORBA_tk_void, 0, 0, -1, 0, 0, NULL,
+	CORBA_OBJECT_NIL, "void", "IDL:omg.org/CORBA/void:1.0"
 };
 
 DEF_TC_BASIC(char, ORBIT_ALIGNOF_CORBA_CHAR);
@@ -278,7 +278,7 @@ CDR_get (GIOPRecvBuffer *buf,
 {
 	buf->cur = ALIGN_ADDRESS (buf->cur, len);
 
-	if ((buf->cur + len) > buf->end || len < 0)
+	if ((buf->cur + len) > buf->end)
 		return TRUE;
 
 	memcpy (ptr, buf->cur, len);
@@ -702,6 +702,297 @@ CORBA_TypeCode_equivalent (CORBA_TypeCode obj,
 	return typecode_equiv_internal (obj, tc, FALSE, ev);
 }
 
+CORBA_TypeCode
+CORBA_TypeCode_get_compact_typecode (CORBA_TypeCode     typecode,
+				     CORBA_Environment *ev)
+{
+	/* FIXME: implement */
+	CORBA_exception_set_system (
+		ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
+
+	return CORBA_OBJECT_NIL;
+}
+
+CORBA_TCKind
+CORBA_TypeCode_kind (CORBA_TypeCode     typecode,
+		     CORBA_Environment *ev)
+{
+	return typecode->kind;
+}
+
+CORBA_RepositoryId
+CORBA_TypeCode_id (CORBA_TypeCode     typecode,
+		   CORBA_Environment *ev)
+{
+	if (!(typecode->kind == CORBA_tk_objref             ||
+	      typecode->kind == CORBA_tk_value              ||
+	      typecode->kind == CORBA_tk_value_box          ||
+	      typecode->kind == CORBA_tk_abstract_interface ||
+	      typecode->kind == CORBA_tk_native             ||
+	      typecode->kind == CORBA_tk_struct             ||
+	      typecode->kind == CORBA_tk_union              ||
+	      typecode->kind == CORBA_tk_enum               ||
+	      typecode->kind == CORBA_tk_alias              ||
+	      typecode->kind == CORBA_tk_except)) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return NULL;
+	}
+
+	return CORBA_string_dup (typecode->repo_id);
+}
+
+CORBA_Identifier
+CORBA_TypeCode_name (CORBA_TypeCode     typecode,
+		     CORBA_Environment *ev)
+{
+	if (!(typecode->kind == CORBA_tk_objref             ||
+	      typecode->kind == CORBA_tk_struct             ||
+	      typecode->kind == CORBA_tk_union              ||
+	      typecode->kind == CORBA_tk_enum               ||
+	      typecode->kind == CORBA_tk_alias              ||
+	      typecode->kind == CORBA_tk_abstract_interface ||
+	      typecode->kind == CORBA_tk_value              ||
+	      typecode->kind == CORBA_tk_value_box          ||
+	      typecode->kind == CORBA_tk_native             ||
+	      typecode->kind == CORBA_tk_except)) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return NULL;
+	}
+
+	return CORBA_string_dup (typecode->name);
+}
+
+CORBA_unsigned_long
+CORBA_TypeCode_member_count (CORBA_TypeCode     typecode,
+			     CORBA_Environment *ev)
+{
+	if (!(typecode->kind == CORBA_tk_struct ||
+	      typecode->kind == CORBA_tk_union  ||
+	      typecode->kind == CORBA_tk_value  ||
+	      typecode->kind == CORBA_tk_enum   ||
+	      typecode->kind == CORBA_tk_except)) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return 0;
+	}
+
+	return typecode->sub_parts;
+}
+
+CORBA_Identifier
+CORBA_TypeCode_member_name (CORBA_TypeCode       typecode,
+			    CORBA_unsigned_long  index,
+			    CORBA_Environment   *ev)
+{
+	if (!(typecode->kind == CORBA_tk_struct ||
+	      typecode->kind == CORBA_tk_union  ||
+	      typecode->kind == CORBA_tk_value  ||
+	      typecode->kind == CORBA_tk_enum   ||
+	      typecode->kind == CORBA_tk_except)) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return NULL;
+	}
+
+	if (index > typecode->sub_parts) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/Bounds/1.0", NULL);
+		return NULL;
+	}
+
+	return CORBA_string_dup (typecode->subnames [index]);
+}
+
+CORBA_TypeCode
+CORBA_TypeCode_member_type (CORBA_TypeCode       typecode,
+			    CORBA_unsigned_long  index,
+			    CORBA_Environment   *ev)
+{
+	if (!(typecode->kind == CORBA_tk_struct ||
+	      typecode->kind == CORBA_tk_union  ||
+	      typecode->kind == CORBA_tk_value  ||
+	      typecode->kind == CORBA_tk_enum   ||
+	      typecode->kind == CORBA_tk_except)) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return NULL;
+	}
+
+	if (index > typecode->sub_parts) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/Bounds/1.0", NULL);
+		return NULL;
+	}
+
+	return ORBit_RootObject_duplicate (typecode->subtypes [index]);
+}
+
+CORBA_any *
+CORBA_TypeCode_member_label (CORBA_TypeCode       typecode,
+			     CORBA_unsigned_long  index,
+			     CORBA_Environment   *ev)
+{
+	CORBA_any *retval;
+
+	if (typecode->kind != CORBA_tk_union) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return NULL;
+	}
+
+	if (index > typecode->sub_parts) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/Bounds/1.0", NULL);
+		return NULL;
+	}
+
+	retval = CORBA_any__alloc ();
+
+	retval->_type  = ORBit_RootObject_duplicate (typecode->discriminator);
+	retval->_value = ORBit_copy_value (&typecode->sublabels [index],
+					   typecode->discriminator);
+	retval->_release = CORBA_TRUE;
+
+	return retval;
+}
+
+CORBA_TypeCode
+CORBA_TypeCode_discriminator_type (CORBA_TypeCode     typecode,
+				   CORBA_Environment *ev)
+{
+	if (typecode->kind != CORBA_tk_union) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return NULL;
+	}
+
+	return ORBit_RootObject_duplicate (typecode->discriminator);
+}
+
+CORBA_long
+CORBA_TypeCode_default_index (CORBA_TypeCode     typecode,
+			      CORBA_Environment *ev)
+{
+	if (typecode->kind != CORBA_tk_union) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return 0;
+	}
+
+	return typecode->default_index;
+}
+
+CORBA_unsigned_long
+CORBA_TypeCode_length (CORBA_TypeCode     typecode,
+		       CORBA_Environment *ev)
+{
+	if (!(typecode->kind == CORBA_tk_string   ||
+	      typecode->kind == CORBA_tk_wstring  ||
+	      typecode->kind == CORBA_tk_sequence ||
+	      typecode->kind == CORBA_tk_array)) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return 0;
+	}
+
+	return typecode->length;
+}
+
+CORBA_TypeCode
+CORBA_TypeCode_content_type (CORBA_TypeCode     typecode,
+			     CORBA_Environment *ev)
+{
+	if (!(typecode->kind == CORBA_tk_sequence  ||
+	      typecode->kind == CORBA_tk_array     ||
+	      typecode->kind == CORBA_tk_value_box ||
+	      typecode->kind == CORBA_tk_alias)) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return NULL;
+	}
+
+	g_assert (typecode->sub_parts == 1);
+
+	return ORBit_RootObject_duplicate (typecode->subtypes [0]);
+}
+
+CORBA_unsigned_short
+CORBA_TypeCode_fixed_digits (CORBA_TypeCode     typecode,
+			     CORBA_Environment *ev)
+{
+	if (typecode->kind != CORBA_tk_fixed) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return 0;
+	}
+
+	return typecode->digits;
+}
+
+CORBA_short
+CORBA_TypeCode_fixed_scale (CORBA_TypeCode     typecode,
+			    CORBA_Environment *ev)
+{
+	if (typecode->kind != CORBA_tk_fixed) {
+		CORBA_exception_set (
+			ev, CORBA_USER_EXCEPTION,
+			"IDL:omg.org/CORBA/TypeCode/BadKind/1.0", NULL);
+		return 0;
+	}
+
+	return typecode->scale;
+}
+
+CORBA_Visibility
+CORBA_TypeCode_member_visibility (CORBA_TypeCode             typecode,
+				  const CORBA_unsigned_long  index,
+				  CORBA_Environment         *ev)
+{
+	/* FIXME: implement */
+	CORBA_exception_set_system (
+		ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
+
+	return 0;
+}
+
+CORBA_ValueModifier
+CORBA_TypeCode_type_modifier (CORBA_TypeCode     typecode,
+			      CORBA_Environment *ev)
+{
+	/* FIXME: implement */
+	CORBA_exception_set_system (
+		ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
+
+	return 0;
+}
+
+CORBA_TypeCode
+CORBA_TypeCode_concrete_base_type (CORBA_TypeCode     typecode,
+				   CORBA_Environment *ev)
+{
+	/* FIXME: implement */
+	CORBA_exception_set_system (
+		ev, ex_CORBA_NO_IMPLEMENT, CORBA_COMPLETED_NO);
+
+	return CORBA_OBJECT_NIL;
+}
+
 const char *
 ORBit_tk_to_name (CORBA_unsigned_long tk)
 {
@@ -779,7 +1070,7 @@ static const TkInfo tk_info[CORBA_tk_last]= {
 	{ TK_SIMPLE, tc_enc_tk_fixed, tc_dec_tk_fixed} /* tk_fixed */
 };
 
-const ORBit_RootObject_Interface ORBit_TypeCode_epv = {
+ORBit_RootObject_Interface ORBit_TypeCode_epv = {
 	ORBIT_ROT_TYPECODE,
 	ORBit_TypeCode_free_fn
 };
